==> substrate-120240315.0.1.tar.gz: substrate-120240315.0.1/substrate/core/corenode.py <==
"""
CORE ꩜ SUBSTRATE
"""
from typing import List, Optional

import networkx as nx

from .base_future import BaseFuture, TraceDirective
from .id_generator import IDGenerator
from .client.future import TracedFuture
from .client.find_futures_client import find_futures_client


class CoreNode:
    def __init__(self, **attr):
        self.node = None
        self.args = attr
        class_name = self.__class__.__name__
        generator_instance = IDGenerator.get_instance(class_name)
        self.id = generator_instance.get_next_id()
        self._global_output_keys = None
        self._should_output_globally: bool = False
        self.SG = nx.DiGraph()
        if attr:
            self.args = BaseFuture.replace_futures_with_placeholder(attr)
        else:
            self.args = {}
        self.SG.add_node(self, **self.args)
        self.futures_from_args: List[BaseFuture] = find_futures_client(attr)

    def to_dict(self):
        return {
            "id": self.id,
            "node": self.node,
            "args": self.args,
            **({"_should_output_globally": self._should_output_globally} if self._should_output_globally else {}),
            **({"_global_output_keys": self._global_output_keys} if self._global_output_keys else {}),
        }

    def subscribe(self, keys: Optional[List[str]] = None):
        """
        Subscribe to the output of this node.
        """
        self._should_output_globally = True
        if keys:
            self._global_output_keys = keys
        return self

    @property
    def future(self) -> TracedFuture:
        """
        Reference to future output of this node.
        """
        return TracedFuture(directive=TraceDirective(op_stack=[], origin_node_id=self.id))

    def __repr__(self):
        return f"{self.__class__.__name__}({self.id})"

    def __hash__(self):
        return hash(self.id)

    def __eq__(self, other):
        return isinstance(other, CoreNode) and self.id == other.id

==> substrate-220240617.0.2.tar.gz: substrate-220240617.0.2/substrate/core/corenode.py <==
"""
CORE ꩜ SUBSTRATE

NOTE: this file is not copied from the main repo
"""
from typing import Any, List, Type, Generic, TypeVar, Optional

import networkx as nx

from .base_future import BaseFuture
from .id_generator import IDGenerator
from .client.future import TracedFuture
from .future_directive import TraceDirective
from .client.find_futures_client import find_futures_client

OT = TypeVar("OT")


class CoreNode(Generic[OT]):
    def __init__(self, out_type: Type[OT] = Type[Any], hide: bool = True, **attr):
        self._out_type = out_type
        self.node = self.__class__.__name__
        self.args = attr
        generator_instance = IDGenerator.get_instance(self.__class__.__name__)
        self.id = generator_instance.get_next_id()
        self._global_output_keys = None
        self._should_output_globally: bool = not hide
        self.SG = nx.DiGraph()
        if attr:
            self.args = BaseFuture.replace_futures_with_placeholder(attr)
        else:
            self.args = {}
        self.SG.add_node(self, **self.args)
        self.futures_from_args: List[BaseFuture] = find_futures_client(attr)
        self.referenced_nodes = [
            future.directive.origin_node for future in self.futures_from_args if isinstance(future, TracedFuture)
        ]

    @property
    def out_type(self) -> Type[OT]:
        """
        The output type of this node.
        """
        return self._out_type

    def to_dict(self):
        return {
            "id": self.id,
            "node": self.node,
            "args": self.args,
            **({"_should_output_globally": self._should_output_globally} if self._should_output_globally else {}),
            **({"_global_output_keys": self._global_output_keys} if self._global_output_keys else {}),
        }

    def subscribe(self, keys: Optional[List[str]] = None):
        self._should_output_globally = True
        if keys:
            self._global_output_keys = keys
        return self

    @property
    def future(self) -> TracedFuture:
        """
        Reference to future output of this node.
        """
        return TracedFuture(directive=TraceDirective(op_stack=[], origin_node=self))

    def __repr__(self):
        return f"{self.__class__.__name__}({self.id})"

    def __hash__(self):
        return hash(self.id)

    def __eq__(self, other):
        return isinstance(other, CoreNode) and self.id == other.id
